/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "constants.h"


#define AUX_W0			16

// ***************************************************************************
// 
// ***************************************************************************
static void aux_insert(size_t n, dict_index *d, uint64_t *v)
{
	for (size_t i = 1; i < n; i++) {
		dict_index p = d[i];
		uint64_t pc = v[p];
		
		size_t j = i;
		while ((j) && (pc < v[d[j - 1]])) {
			d[j] = d[j - 1];
			j--;
		}
		
		d[j] = p;
	}
}

// ***************************************************************************
// 
// ***************************************************************************
static void aux_merge(size_t w0, size_t w1, dict_index *a, dict_index *d,
	uint64_t *v)
{
	dict_index *d0 = d;
	dict_index *d1 = d + w0;
	
	dict_index p0 = *d0;
	dict_index p1 = *d1;
	uint64_t p0c = v[p0];
	uint64_t p1c = v[p1];
	
	while (1) {
		if (p1c < p0c) {
			*(a++) = p1;
			
			if (!(--w1))
				break;

			p1 = *(++d1);
			p1c = v[p1];
		} else {
			*(a++) = p0;
			
			if (!(--w0))
				break;
			
			p0 = *(++d0);
			p0c = v[p0];
		}
	}
	
	if (w0) {
		*(a++) = p0;
		while (--w0)
			*(a++) = *(++d0);
	} else if (w1) {
		*(a++) = p1;
		while (--w1)
			*(a++) = *(++d1);
	}
}

// ***************************************************************************
// 
// ***************************************************************************
static int aux_sort_swapper(size_t n, dict_index *d, dict_index *a,
	uint64_t *v)
{
	size_t o;
	for (o = 0; o + AUX_W0 <= n; o += AUX_W0)
		aux_insert(AUX_W0, d + o, v);
	
	if (o + 1 < n)
		aux_insert(n - o, d + o, v);
	
	int odd = 0;
	
	for (size_t w = AUX_W0; w < n; w *= 2) {
		for (o = 0; o + (w * 2) <= n; o += (w * 2))
			aux_merge(w, w, a + o, d + o, v);
		
		if (o + w < n) {
			aux_merge(w, n - o - w, a + o, d + o, v);
		} else {
			for (; o < n; o++)
				a[o] = d[o];
		}
		
		dict_index *t = a;
		a = d;
		d = t;
		odd = !odd;
	}
	
	return odd;
}

// ***************************************************************************
// 
// ***************************************************************************
void aux_sort(size_t n, dict_index **d, dict_index **a, uint64_t *v)
{
	if (aux_sort_swapper(n, *d, *a, v)) {
		dict_index *t = *d;
		*d = *a;
		*a = t;
	}
}
