/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef CONSTANTS_H
#define CONSTANTS_H
#include <stddef.h>
#include <stdint.h>


// ****************************************************************************
// Hardcoded constants and types
// ****************************************************************************
/*
  [dict] LENGTH:
    length of a word
  
  [lut] THREE_TO_LENGTH:
    3 ^ LENGTH
  
  [lut] hash_index:
    must accomodate integers in { 0, ..., (THREE_TO_LENGTH - 1) }
  
  [lut] DICT_SIZE:
    size of dictionary
  
  [lut] DICT_SIZE_SHIFT:
    ceil(log2(DICT_SIZE))
  
  [lut] dict_index:
    must accomodate integers in { 0, ..., (DICT_SIZE - 1) }
  
  Notes:
  
    longest word length in /usr/share/dict/words is currently 45
    official https://www.powerlanguage.co.uk/wordle/ is currently 5
*/
#define LENGTH			((hash_index)5)
#define THREE_TO_LENGTH		((hash_index)243)
#define DISAMBIGUATION		((hash_index)212)
typedef uint8_t			hash_index;

#define HASH_ANY		((hash_index)-2)
#define HASH_NONE		((hash_index)-1)
#define HASH_ALL_MATCH		(THREE_TO_LENGTH - 1)

#define DICT_SIZE		((dict_index)12972)
#define DICT_NONE		((dict_index)-1)
typedef uint16_t		dict_index;

// ****************************************************************************
// Options
// ****************************************************************************
/*
  CHEAT_MODE:
    Consider that targets can be take only from the solution list.
    
  HARD_MODE:
    Wordle hard mode - one can play only words that fit the known
    constraints (although with no additional inference).
*/
#define GAME_REGULAR	0
#define GAME_CHEAT	1
#define GAME_HARD	2



// ****************************************************************************
// Compiler tools
// ****************************************************************************
#define FN_INLINE     __attribute__ ((unused)) static inline


// ****************************************************************************
// 
// ****************************************************************************

#endif

