/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef DICT_H
#define DICT_H
#include <stddef.h>
#include <stdint.h>
#include "constants.h"
#include "word.h"

// ****************************************************************************
// 
// ****************************************************************************
#define DICT_ALLOC_MIN	512

typedef struct dict_t {
	size_t count, alloc;
	word *words;
} dict;


// ****************************************************************************
// 
// ****************************************************************************
void dict_init(dict *d);
void dict_clear(dict *d);

FN_INLINE void dict_reset(dict *d)
{
	d->count = 0;
}


void dict_grow_nt(dict *d, size_t xc);
FN_INLINE void dict_grow(dict *d, size_t xc)
{
	if (xc > d->alloc)
		dict_grow_nt(d, xc);
}

FN_INLINE void dict_append_word(dict *d, word w)
{
	size_t dc = d->count;
	
	dict_grow(d, dc + 1);
	
	d->words[dc] = w;
	
	d->count = dc + 1;
}

void dict_append_dict(dict *dst, const dict *src);
void dict_append_file(dict *d, const char *path);

void dict_lower(dict *d);
void dict_sort(dict *d);

void dict_filter_dupes(dict *dst, const dict *src);
void dict_filter_letters(dict *dst, const dict *src);
void dict_filter_pattern(dict *dst, const dict *src, word fix, lset contain);
void dict_subtract(dict *dst, const dict *a, const dict *b);

dict_index dict_index_get(const dict *d, const word w);
int dict_has(const dict *d, const word w);

// ****************************************************************************
// 
// ****************************************************************************
#endif

