/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef LOOKUP_H
#define LOOKUP_H
#include <stdio.h>
#include "constants.h"
#include "dict.h"

// ****************************************************************************
// 
// ****************************************************************************
typedef struct mask_t {
	lset fix[LENGTH];
	lset contain;
} mask;

// 150+ MB, do not attempt to allocate on stack.
typedef struct lut_t {
	hash_index hash[DICT_SIZE][DICT_SIZE];
	mask word_mask[DICT_SIZE];
	lset fix[THREE_TO_LENGTH][LENGTH];
	lset contain[THREE_TO_LENGTH][LENGTH];
	word hash_word[THREE_TO_LENGTH];
} lut;

void lut_build(lut *t, dict *d);

hash_index lut_hash_parse(word a);

word lut_hash_word(hash_index h);
void lut_print_hash(FILE *f, hash_index h);

FN_INLINE hash_index lut_lookup(lut *t, dict_index guess, dict_index target)
{
	return t->hash[guess][target];
}

FN_INLINE hash_index *lut_lookup_guess(lut *t, dict_index guess)
{
	return t->hash[guess];
}

// ****************************************************************************
// 
// ****************************************************************************
int hard_allowed(lut *t,
	word prev, hash_index prev_hash,
	word guess);


// ****************************************************************************
// 
// ****************************************************************************
#endif

