/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef WORD_H
#define WORD_H
#include <stddef.h>
#include <stdint.h>
#include "constants.h"

// ****************************************************************************
// 
// ****************************************************************************
typedef union word_t {
	uint8_t s[LENGTH];
} word;

typedef uint32_t lset;

#define LSET_ZERO	((lset)0)
#define LSET_ONE	((lset)1)
#define LSET_ALL	(((lset)1 << 26) - 1)

// ****************************************************************************
// 
// ****************************************************************************
FN_INLINE word word_zero(void)
{
	word w;
	__builtin_memset(&w, 0, sizeof(word));
	return w;
}

FN_INLINE int word_is(word w0, word w1)
{
	return (__builtin_memcmp(&w0.s, &w1.s, sizeof(word)) == 0);
}

FN_INLINE int word_has(word w, uint8_t c)
{
	int r = 0;

	for (size_t j = 0; j < LENGTH; j++)
		r |= (w.s[j] == c);
	
	return r;
}

FN_INLINE lset lset_one(uint8_t c)
{
	return (((lset)1) << (c - 'a'));
}

FN_INLINE int lset_has(lset s, uint8_t c)
{
	return ((s & lset_one(c)) != 0);
}

FN_INLINE lset lset_build(word w)
{
	lset s = 0;
	
	for (size_t j = 0; j < LENGTH; j++) {
		uint8_t c = w.s[j];
		
		s |= lset_one(c);
	}
	
	return s;
}

FN_INLINE int word_has_all(word w, lset s)
{
	lset ws = lset_build(w);
	
	return ((s & (~ws)) == 0);
}

FN_INLINE int word_fix_match(word w, word f)
{
	int r = 1;
	
	for (size_t j = 0; j < LENGTH; j++) {
		if ((f.s[j] != 0) && (f.s[j] != w.s[j]))
			r = 0;
	}
	
	return r;
}

FN_INLINE int word_cmp(word w0, word w1)
{
	return __builtin_memcmp(w0.s, w1.s, LENGTH);
}

word word_build(const char *s);
word word_fix(const char *fix);
word word_lower(word w);
int word_letters(word w);

FN_INLINE struct small_str_t {
	char s[LENGTH + 1];
} word_str(word w)
{
	struct small_str_t s;
	
	__builtin_memcpy(s.s, w.s, LENGTH);
	s.s[LENGTH] = 0;
	
	return s;
}



// ****************************************************************************
// 
// ****************************************************************************
#endif

